/*
 * csv.c
 *
 * author: Roberto Piassi Passos Bodo
 *  email: rppbodo@gmail.com
 */

#include "csv.h"

/*!
 * This function receives a CSV file, reads its bytes, interprets them and
 * fills the table of MIDI events. It also assigns values to some important
 * global variables (like 'n_tracks' and 'ticks_per_beat'). It returns 0 in
 * case of success and returns 1 otherwise.
 */
int load_csv(FILE* input) {
	char buffer[MAX_C];
	char line[MAX_C];

	int first = 1;

	load_instruments();

	while (!feof(input)) {
		if (fgets(buffer, MAX_C, input) != NULL) {
			int track;
			int delta_time;

			strcpy(line, buffer);

			track = atoi(strtok(buffer, ", "));

			if (track == 0 && first) {
				int k;

				for (k = 0; k < 5; k++) {
					if (k == 2) {
						if (atoi(strtok(NULL, ", ")) != 1) {
							fprintf(stderr, "error: csv file is not valid...\n");
							return 1;
						}
					} else if (k == 3) {
						n_tracks = atoi(strtok(NULL, ", "));
					} else if (k == 4) {
						ticks_per_beat = atoi(strtok(NULL, ", "));
					} else {
						strtok(NULL, ", ");
					}
				}

				init_table();

				first = 0;
			} else {
				char temp[50];

				delta_time = atoi(strtok(NULL, ", "));

				strcpy(temp, strtok(NULL, ", "));

				if (strcmp(temp, "Tempo") == 0) {
					int tempo = atoi(strtok(NULL, ", "));
					insert_tempo(delta_time, tempo);
				} else {
					if (strcmp(temp, "Note_off_c") == 0) {
						int midi_channel = atoi(strtok(NULL, ", "));
						int note = atoi(strtok(NULL, ", "));
						int velocity = atoi(strtok(NULL, ", "));

						unsigned char bytes[3];

						bytes[0] = 128 + midi_channel;
						bytes[1] = note;
						bytes[2] = velocity;

						insert_table(track, delta_time, bytes);
					} else if (strcmp(temp, "Note_on_c") == 0) {
						int midi_channel = atoi(strtok(NULL, ", "));
						int note = atoi(strtok(NULL, ", "));
						int velocity = atoi(strtok(NULL, ", "));

						unsigned char bytes[3];

						bytes[0] = 144 + midi_channel;
						bytes[1] = note;
						bytes[2] = velocity;

						insert_table(track, delta_time, bytes);
					} else if (strcmp(temp, "Control_c") == 0) {
						int midi_channel = atoi(strtok(NULL, ", "));
						int type = atoi(strtok(NULL, ", "));
						int value = atoi(strtok(NULL, ", "));

						unsigned char bytes[3];

						bytes[0] = 176 + midi_channel;
						bytes[1] = type;
						bytes[2] = value;

						insert_table(track, delta_time, bytes);
					} else if (strcmp(temp, "Program_c") == 0) {
						int midi_channel = atoi(strtok(NULL, ", "));
						int instrument = atoi(strtok(NULL, ", "));

						unsigned char bytes[3];

						bytes[0] = 192 + midi_channel;
						bytes[1] = instrument;
						bytes[2] = -1;

						printf("track #%d has: %s\n", track,
								instruments[instrument]);

						write(output_device_fd, bytes, 2);
					} else if (strcmp(temp, "Pitch_bend_c") == 0) {
						int midi_channel = atoi(strtok(NULL, ", "));
						int x = atoi(strtok(NULL, ", "));

						unsigned char bytes[3];

						bytes[0] = 224 + midi_channel;
						bytes[1] = (x & 0x7f);
						bytes[2] = (x & 0x3f80) >> 7;

						insert_table(track, delta_time, bytes);
					}
				}
			}
		}
	}

	return 0;
}
